# -*- coding: utf-8 -*-
"""clip.py - Page Clip Setting
"""

import gtk

from preferences import prefs

_dialog = None


class _PageClipDialog(gtk.Dialog):

    """A gtk.Dialog Page Clip Setting.
    """

    def __init__(self, window):
        self._window = window
        
        gtk.Dialog.__init__(self, _('Clip setting'), window, 0)
        self.add_buttons(_('Auto'), gtk.RESPONSE_APPLY, _('Defaults'), gtk.RESPONSE_NO, gtk.STOCK_CLOSE, gtk.RESPONSE_OK)
        self.set_has_separator(False)
        self.set_resizable(False)
        self.connect('response', self._response)
        self.set_default_response(gtk.RESPONSE_OK)
        
        self.set_size_request(400, 240)

        self._block = False

        vbox = gtk.VBox(False, 10)
        self.set_border_width(4)
        vbox.set_border_width(6)
        self.vbox.add(vbox)

        hbox = gtk.HBox(False, 4)
        vbox.pack_start(hbox, False, False, 2)
        vbox_left = gtk.VBox(False, 4)
        vbox_right = gtk.VBox(False, 4)
        hbox.pack_start(vbox_left, False, False, 2)
        hbox.pack_start(vbox_right, True, True, 2)

        label = gtk.Label(_('Left') + ':')
        label.set_alignment(1, 0.5)
        vbox_left.pack_start(label, True, False, 2)
        adj = gtk.Adjustment(0, -25, 25, 0.5, 1)
        self._left_scale = gtk.HScale(adj)
        self._left_scale.set_digits(1)
        self._left_scale.set_value_pos(gtk.POS_RIGHT)
        self._left_scale.connect('value-changed', self._change_values)
        self._left_scale.set_update_policy(gtk.UPDATE_DELAYED)
        vbox_right.pack_start(self._left_scale, True, False, 2)

        label = gtk.Label(_('Top') + ':')
        label.set_alignment(1, 0.5)
        vbox_left.pack_start(label, True, False, 2)
        adj = gtk.Adjustment(0, -25, 25, 0.5, 1)
        self._top_scale = gtk.HScale(adj)
        self._top_scale.set_digits(1)
        self._top_scale.set_value_pos(gtk.POS_RIGHT)
        self._top_scale.connect('value-changed', self._change_values)
        self._top_scale.set_update_policy(gtk.UPDATE_DELAYED)
        vbox_right.pack_start(self._top_scale, True, False, 2)

        label = gtk.Label(_('Right') + ':')
        label.set_alignment(1, 0.5)
        vbox_left.pack_start(label, True, False, 2)
        adj = gtk.Adjustment(0, -25, 25, 0.5, 1)
        self._right_scale = gtk.HScale(adj)
        self._right_scale.set_digits(1)
        self._right_scale.set_value_pos(gtk.POS_RIGHT)
        self._right_scale.connect('value-changed', self._change_values)
        self._right_scale.set_update_policy(gtk.UPDATE_DELAYED)
        vbox_right.pack_start(self._right_scale, True, False, 2)

        label = gtk.Label(_('Bottom') + ':')
        label.set_alignment(1, 0.5)
        vbox_left.pack_start(label, True, False, 2)
        adj = gtk.Adjustment(0, -25, 25, 0.5, 1)
        self._bottom_scale = gtk.HScale(adj)
        self._bottom_scale.set_digits(1)
        self._bottom_scale.set_value_pos(gtk.POS_RIGHT)
        self._bottom_scale.connect('value-changed', self._change_values)
        self._bottom_scale.set_update_policy(gtk.UPDATE_DELAYED)
        vbox_right.pack_start(self._bottom_scale, True, False, 2)

        vbox.pack_start(gtk.HSeparator(), True, False, 4)

        self._enabled_button = gtk.CheckButton(_('Clip enabled'))
        vbox.pack_start(self._enabled_button, True, False, 4)
        self._enabled_button.connect('toggled', self._change_enabled)
        
        self._double_enabled_button = gtk.CheckButton(_('Clip enabled in double page'))
        vbox.pack_start(self._double_enabled_button, False, False, 0)
        self._double_enabled_button.connect('toggled', self._change_double_enabled)

        self._block = True
        self._left_scale.set_value(prefs['clip_left'])
        self._top_scale.set_value(prefs['clip_top'])
        self._right_scale.set_value(prefs['clip_right'])
        self._bottom_scale.set_value(prefs['clip_bottom'])
        self._enabled_button.set_active(prefs['clip_enabled'])
        self._double_enabled_button.set_active(prefs['clip_double_enabled'])
        self._block = False

        self.show_all()

    def _change_values(self, *args):
        if self._block: return
        prefs['clip_left'] = self._left_scale.get_value()
        prefs['clip_top'] = self._top_scale.get_value()
        prefs['clip_right'] = self._right_scale.get_value()
        prefs['clip_bottom'] = self._bottom_scale.get_value()
        if prefs['clip_enabled']:
            if (self._window.is_double_page):
                if prefs['clip_double_enabled']: self._window.draw_image()
            else:
                self._window.draw_image()

    def _change_enabled(self, *args):
        if self._block: return
        prefs['clip_enabled'] = self._enabled_button.get_active()
        self._window.actiongroup.get_action('clip_enabled').set_active(prefs['clip_enabled'])

    def _change_double_enabled(self, *args):
        if self._block: return
        prefs['clip_double_enabled'] = self._double_enabled_button.get_active()
        #self._window.actiongroup.get_action('clip_double_enabled').set_active(prefs['clip_double_enabled'])
        if prefs['clip_enabled'] and self._window.is_double_page: self._window.draw_image()

    def _response(self, dialog, response):
        if response in [gtk.RESPONSE_OK, gtk.RESPONSE_DELETE_EVENT]:
            _close_dialog()
        elif response == gtk.RESPONSE_NO:
            self._left_scale.set_value(0)
            self._top_scale.set_value(0)
            self._right_scale.set_value(0)
            self._bottom_scale.set_value(0)
            self._change_values(self)
        elif response == gtk.RESPONSE_APPLY:
            clip = self._window.get_auto_clip_pixels()
            self._left_scale.set_value(clip[0])
            self._top_scale.set_value(clip[1])
            self._right_scale.set_value(clip[2])
            self._bottom_scale.set_value(clip[3])
            self._change_values(self)

def open_dialog(action, window):
    """Create and display the (singleton) image enhancement dialog."""
    global _dialog
    if _dialog is None:
        _dialog = _PageClipDialog(window)
    else:
        _dialog.present()


def _close_dialog(*args):
    """Destroy the image enhancement dialog."""
    global _dialog
    if _dialog is not None:
        _dialog.destroy()
        _dialog = None
